﻿using System.Collections;
using System.Collections.Generic;
using System.Data;
using Vevo.DataAccessLib;
using Vevo.Domain;
using Vevo.Domain.Base;
using Vevo.Domain.DataInterfaces;
using Vevo.Domain.Products;
using Vevo.Shared.DataAccess;
using Vevo.Shared.Utilities;
using Vevo.Base.Domain;

namespace Vevo.Data
{
    public class DepartmentRepositoryNoCache : IDepartmentRepository
    {
        #region Private
        private const string AllDepartmentFields =
            "Department.DepartmentID, ImageFile, ParentDepartmentID, Department.IsEnabled, " +
            "SortOrder, DiscountGroupID, Other1, Other2, Other3, Other4, Other5, " +
            "DepartmentListLayoutPath, ProductListLayoutPath, RootID, IsAnchor, IsShowNewArrival, NewArrivalAmount ";

        private const string AllDepartmentLocaleFields =
            "DepartmentID, CultureID, Name, Description, MetaKeyword, MetaDescription, ImageAlt, ImageTitle, UrlName ";

        private const string AllDepartmentFieldsWithLocale =
            "Department.DepartmentID, Name, Description, ImageFile, ParentDepartmentID, UrlName, Department.IsEnabled, " +
            "SortOrder, DiscountGroupID, Other1, Other2, Other3, Other4, Other5, IsAnchor, IsShowNewArrival, NewArrivalAmount, " +
            "DepartmentListLayoutPath, ProductListLayoutPath, RootID, MetaKeyword, MetaDescription, ImageAlt, ImageTitle ";

        private static UrlNameAccessHelper _urlNameHelper = new UrlNameAccessHelper( "DepartmentLocale", "DepartmentID" );
        private CategoryHelperRepository _categoryHelperRepository = new CategoryHelperRepository( "Department" );

        private static string ConvertFieldName( string fieldName )
        {
            switch (fieldName.ToLower())
            {
                case "departmentid":
                    return "Department.DepartmentID";

                default:
                    return fieldName;
            }
        }

        private string CreateSortByText( string sortBy )
        {
            if (sortBy == null || sortBy.Trim() == "")
                return "";
            else if (sortBy.Trim().ToLower().StartsWith( "departmentid" ))
                return "ORDER BY Department." + sortBy.Trim() + " ";
            else if (sortBy.Trim().ToLower().StartsWith( "name" ))
                return "ORDER BY L." + sortBy.Trim() + " ";
            else if (sortBy.Trim().ToLower().StartsWith( "sortorder" ))
                return "ORDER BY Department." + sortBy.Trim() + " ";
            else
                return "ORDER BY " + sortBy + " ";
        }

        private string GetCultureID( Culture culture )
        {
            return culture.CultureID;
        }

        private void DeleteDepartmentLocales( string departmentID )
        {
            DataAccess.ExecuteNonQuery(
                "DELETE FROM DepartmentLocale " +
                "WHERE DepartmentID = @DepartmentID; ",
                DataAccess.CreateParameterString( departmentID ) );
        }

        private void InsertDepartmentLocales( Department department )
        {
            foreach (BaseCategoryLocale locale in department.Locales)
            {
                locale.UrlName = _urlNameHelper.CreateLocaleUrlName( locale.Name );
                DataAccess.ExecuteNonQuery(
                    "INSERT INTO DepartmentLocale ( DepartmentID, CultureID, Name, Description, MetaKeyword, MetaDescription, ImageAlt, ImageTitle, UrlName ) " +
                    "VALUES ( @DepartmentID, @CultureID, @Name, @Description, @MetaKeyword, @MetaDescription, @ImageAlt, @ImageTitle, @UrlName ); " +
                    "SELECT @@Identity; ",
                    DataAccess.CreateParameterString( department.DepartmentID ),
                    DataAccess.CreateParameterString( locale.CultureID ),
                    DataAccess.CreateParameterString( locale.Name ),
                    DataAccess.CreateParameterString( locale.Description ),
                    DataAccess.CreateParameterString( locale.MetaKeyword ),
                    DataAccess.CreateParameterString( locale.MetaDescription ),
                    DataAccess.CreateParameterString( locale.ImageAlt ),
                    DataAccess.CreateParameterString( locale.ImageTitle ),
                    DataAccess.CreateParameterString( locale.UrlName ) );
            }
        }

        private Department Create( Department department )
        {
            int recCount = int.Parse( DataAccess.ExecuteScalar(
                "SELECT COUNT(*) FROM Department WHERE ParentDepartmentID = @ParentDepartmentID and RootID = @RootID;",
                DataAccess.CreateParameterString( department.ParentDepartmentID ),
                DataAccess.CreateParameterString( department.RootID ) ) );

            string newID = DataAccess.ExecuteScalar(
                "INSERT INTO Department ( ImageFile, ParentDepartmentID, SortOrder, DiscountGroupID, IsEnabled, " +
                " Other1, Other2, Other3, Other4, Other5, DepartmentListLayoutPath, ProductListLayoutPath, RootID, IsAnchor, IsShowNewArrival, NewArrivalAmount ) " +
                "VALUES ( @Image, @ParentDepartmentID, @SortOrder, @DiscountGroupID, @IsEnabled, " +
                " @Other1, @Other2, @Other3, @Other4, @Other5, @DepartmentListLayoutPath, @ProductListLayoutPath, @RootID, @IsAnchor, @IsShowNewArrival, @NewArrivalAmount); " +
                "SELECT @@Identity; ",
                DataAccess.CreateParameterString( department.ImageFile ),
                DataAccess.CreateParameterString( department.ParentDepartmentID ),
                DataAccess.CreateParameterInt32( recCount++ ),
                DataAccess.CreateParameterInt32( department.DiscountGroupID ),
                DataAccess.CreateParameterBool( department.IsEnabled ),
                DataAccess.CreateParameterString( department.Other1 ),
                DataAccess.CreateParameterString( department.Other2 ),
                DataAccess.CreateParameterString( department.Other3 ),
                DataAccess.CreateParameterString( department.Other4 ),
                DataAccess.CreateParameterString( department.Other5 ),
                DataAccess.CreateParameterString( department.DepartmentListLayoutPath ),
                DataAccess.CreateParameterString( department.ProductListLayoutPath ),
                DataAccess.CreateParameterString( department.RootID ),
                DataAccess.CreateParameterBool( department.IsAnchor ),
                DataAccess.CreateParameterBool( department.IsShowNewArrival ),
                DataAccess.CreateParameterInt32( department.NewArrivalAmount )
                );

            department.DepartmentID = newID;


            InsertDepartmentLocales( department );

            return department;
        }

        private Department CreateWithIdentity( Department department )
        {
            int recCount = int.Parse( DataAccess.ExecuteScalar(
                "SELECT COUNT(*) FROM Department WHERE ParentDepartmentID = @ParentDepartmentID and RootID = @RootID;",
                DataAccess.CreateParameterString( department.ParentDepartmentID ),
                DataAccess.CreateParameterString( department.RootID ) ) );

            string newID = DataAccess.ExecuteScalar(
                "SET IDENTITY_INSERT [dbo].[Department] ON " +
                "INSERT INTO Department ( DepartmentID, ImageFile, ParentDepartmentID, SortOrder, DiscountGroupID, IsEnabled, " +
                " Other1, Other2, Other3, Other4, Other5, DepartmentListLayoutPath, ProductListLayoutPath, RootID, IsAnchor, IsShowNewArrival, NewArrivalAmount ) " +
                "VALUES ( @DepartmentID, @Image, @ParentDepartmentID, @SortOrder, @DiscountGroupID, @IsEnabled, " +
                " @Other1, @Other2, @Other3, @Other4, @Other5, @DepartmentListLayoutPath, @ProductListLayoutPath, @RootID, @IsAnchor, @IsShowNewArrival, @NewArrivalAmount); " +
                "SET IDENTITY_INSERT [dbo].[Department] OFF ",
                DataAccess.CreateParameterString( department.DepartmentID ),
                DataAccess.CreateParameterString( department.ImageFile ),
                DataAccess.CreateParameterString( department.ParentDepartmentID ),
                DataAccess.CreateParameterInt32( recCount++ ),
                DataAccess.CreateParameterInt32( department.DiscountGroupID ),
                DataAccess.CreateParameterBool( department.IsEnabled ),
                DataAccess.CreateParameterString( department.Other1 ),
                DataAccess.CreateParameterString( department.Other2 ),
                DataAccess.CreateParameterString( department.Other3 ),
                DataAccess.CreateParameterString( department.Other4 ),
                DataAccess.CreateParameterString( department.Other5 ),
                DataAccess.CreateParameterString( department.DepartmentListLayoutPath ),
                DataAccess.CreateParameterString( department.ProductListLayoutPath ),
                DataAccess.CreateParameterString( department.RootID ),
                DataAccess.CreateParameterBool( department.IsAnchor ),
                DataAccess.CreateParameterBool( department.IsShowNewArrival ),
                DataAccess.CreateParameterInt32( department.NewArrivalAmount )
                );

            InsertDepartmentLocales( department );

            return department;
        }

        private Department FindByDepartmentID( IList<Department> allCategories, string departmentID )
        {
            foreach (Department department in allCategories)
            {
                if (department.DepartmentID == departmentID)
                    return department;
            }

            return Department.Null;
        }

        private bool IsChildOf( IList<Department> allCategories, string childID, string parentID )
        {
            string currentID = childID;
            int iNumSearch = 0;

            while (currentID != parentID
                && currentID != "0")
            {
                Department department = FindByDepartmentID( allCategories, currentID );
                if (department.IsNull)
                {
                    throw new DataAccessException(
                        "Cannot find department with a department ID " + currentID + "." );
                }

                iNumSearch++;
                if (iNumSearch > allCategories.Count)
                    throw new DataAccessException(
                        "Invalid parent-child department relationship. Possibly a circular list" );

                currentID = department.ParentDepartmentID.ToString();
            }

            if (currentID == parentID)
                return true;
            else
                return false;
        }

        private Department Update( Department department )
        {
            IList<Department> allCategories = GetAll( department.DefaultCulture, "DepartmentID" );
            if (IsChildOf( allCategories, department.ParentDepartmentID, department.DepartmentID ))
            {
                Department parent = FindByDepartmentID( allCategories, department.ParentDepartmentID );

                throw new DataAccessException(
                    "Department '" +
                    department.Name +
                    "' is already the parent of department '" +
                    parent.Name +
                    "'. Cannot be chosen as the parent." );
            }

            int mainUpdate = DataAccess.ExecuteNonQuery(
                "UPDATE Department " +
                "SET ImageFile = @ImageFile, ParentDepartmentID = @ParentDepartmentID, " +
                "    DiscountGroupID = @DiscountGroupID, IsEnabled = @IsEnabled, " +
                "    Other1 = @Other1, Other2 = @Other2, Other3 = @Other3, Other4 = @Other4, Other5 = @Other5, " +
                "    DepartmentListLayoutPath = @DepartmentListLayoutPath, ProductListLayoutPath = @ProductListLayoutPath , IsAnchor = @IsAnchor, IsShowNewArrival = @IsShowNewArrival, NewArrivalAmount = @NewArrivalAmount " +
                "WHERE DepartmentID = @DepartmentID; ",
                DataAccess.CreateParameterString( department.ImageFile ),
                DataAccess.CreateParameterString( department.ParentDepartmentID ),
                DataAccess.CreateParameterInt32( department.DiscountGroupID ),
                DataAccess.CreateParameterBool( department.IsEnabled ),
                DataAccess.CreateParameterString( department.Other1 ),
                DataAccess.CreateParameterString( department.Other2 ),
                DataAccess.CreateParameterString( department.Other3 ),
                DataAccess.CreateParameterString( department.Other4 ),
                DataAccess.CreateParameterString( department.Other5 ),
                DataAccess.CreateParameterString( department.DepartmentListLayoutPath ),
                DataAccess.CreateParameterString( department.ProductListLayoutPath ),
                DataAccess.CreateParameterBool( department.IsAnchor ),
                DataAccess.CreateParameterBool( department.IsShowNewArrival ),
                DataAccess.CreateParameterInt32( department.NewArrivalAmount ),
                DataAccess.CreateParameterString( department.DepartmentID ) );

            if (mainUpdate == 0)
                throw new DataAccessException(
                    "Department Update Failed. Not Found department with ID: " + department.DepartmentID +
                    ", Name: " + department.Name );


            department.Locales.Load();
            DeleteDepartmentLocales( department.DepartmentID );
            InsertDepartmentLocales( department );

            return department;
        }

        private Department CreateOneDepartmentFromRow( Culture defaultCulture, DataRow row )
        {
            Department department = new Department( defaultCulture );

            // Data from Department database table
            department.DepartmentID = row["DepartmentID"].ToString();
            department.ImageFile = row["ImageFile"].ToString();
            department.ParentDepartmentID = row["ParentDepartmentID"].ToString();
            department.DiscountGroupID = (int) row["DiscountGroupID"];
            department.IsEnabled = (bool) row["IsEnabled"];
            department.Other1 = row["Other1"].ToString();
            department.Other2 = row["Other2"].ToString();
            department.Other3 = row["Other3"].ToString();
            department.Other4 = row["Other4"].ToString();
            department.Other5 = row["Other5"].ToString();
            department.DepartmentListLayoutPath = ConvertUtilities.ToString( row["DepartmentListLayoutPath"] );
            department.ProductListLayoutPath = ConvertUtilities.ToString( row["ProductListLayoutPath"] );
            department.RootID = row["RootID"].ToString();
            department.IsAnchor = (bool) row["IsAnchor"];
            department.IsShowNewArrival = (bool) row["IsShowNewArrival"];
            department.NewArrivalAmount = (int) row["NewArrivalAmount"];

            // Data from DepartmentLocale database table
            department.Name = row["Name"].ToString();
            department.Description = row["Description"].ToString();
            department.MetaKeyword = row["MetaKeyword"].ToString();
            department.MetaDescription = row["MetaDescription"].ToString();
            department.ImageAlt = row["ImageAlt"].ToString();
            department.ImageTitle = row["ImageTitle"].ToString();
            department.UrlName = row["UrlName"].ToString();
            return department;
        }

        private DepartmentLocale CreateOneDepartmentLocaleFromRow( DataRow row )
        {
            DepartmentLocale locale = new DepartmentLocale();
            locale.CultureID = row["CultureID"].ToString();
            locale.Name = row["Name"].ToString();
            locale.Description = row["Description"].ToString();
            locale.MetaKeyword = row["MetaKeyword"].ToString();
            locale.MetaDescription = row["MetaDescription"].ToString();
            locale.ImageAlt = row["ImageAlt"].ToString();
            locale.ImageTitle = row["ImageTitle"].ToString();
            locale.UrlName = row["UrlName"].ToString();
            return locale;
        }

        private List<ILocale> ConvertDepartmentLocaleTableToList( DataTable localeTable )
        {
            List<ILocale> list = new List<ILocale>();

            foreach (DataRow row in localeTable.Rows)
                list.Add( CreateOneDepartmentLocaleFromRow( row ) );

            return list;
        }

        private IList<Department> ConvertDepartmentTableToList( Culture defaultCulture, DataTable table )
        {
            IList<Department> list = new List<Department>();

            foreach (DataRow row in table.Rows)
                list.Add( CreateOneDepartmentFromRow( defaultCulture, row ) );

            return list;
        }

        private IList<Department> GetOnlyIndex( IList<Department> departmentList, int startIndex, int endIndex )
        {
            IList<Department> list = new List<Department>();
            for (int i = startIndex; i <= endIndex && i < departmentList.Count; i++)
            {
                list.Add( departmentList[i] );
            }
            return list;
        }

        #endregion

        #region Constructors

        public DepartmentRepositoryNoCache()
        {

        }

        #endregion

        #region Public Methods

        public Department Save( Department department )
        {
            Department result;
            using (DataTransactionScope transaction = new DataTransactionScope())
            {
                if (department.DepartmentID == "0")
                    result = Create( department );
                else
                    result = Update( department );

                transaction.Complete();
            }

            return result;
        }

        public Department SaveWithIdentity( Department department )
        {
            Department result;
            using (DataTransactionScope transaction = new DataTransactionScope())
            {
                result = CreateWithIdentity( department );

                transaction.Complete();
            }

            return result;
        }

        public void Delete( string departmentID )
        {
            _categoryHelperRepository.Delete( departmentID );
        }

        public void DeleteLocalesByCultureID( string cultureID )
        {
            _categoryHelperRepository.DeleteLocalesByCultureID( cultureID );
        }

        public void ReArrangeSortOrder( Culture culture, string parentDepartmentID, string departmentID, BoolFilter isEnabled )
        {
            IList<Department> list = DataAccessContext.DepartmentRepository.GetByParentID( culture, parentDepartmentID, "SortOrder", isEnabled );
            int recCount = 0;
            string[] result = new string[list.Count + 1];
            foreach (Department department in list)
            {
                result[recCount] = department.DepartmentID;
                recCount++;
            }
            result[recCount] = departmentID;
            DataAccessContext.DepartmentRepository.UpdateSortOrder( result );
        }

        public void ReArrangeSortOrderByParentDepartmentID( Culture culture, string parentDepartmentID, BoolFilter isEnabled )
        {
            IList<Department> list = DataAccessContext.DepartmentRepository.GetByParentID( culture,
                parentDepartmentID, "sortOrder", isEnabled );
            string[] result = new string[list.Count];
            int recCount = 0;
            foreach (Department department in list)
            {
                result[recCount] = department.DepartmentID;
                recCount++;
            }
            DataAccessContext.DepartmentRepository.UpdateSortOrder( result );
        }

        public void UpdateSortOrder( string[] sortedDepartmentIDs )
        {
            _categoryHelperRepository.UpdateSortOrder( sortedDepartmentIDs );
        }

        public Department GetOne( Culture culture, string departmentID )
        {
            DataTable departmentTable = _categoryHelperRepository.GetOne( culture, departmentID );
            if (departmentTable.Rows.Count > 0)
                return CreateOneDepartmentFromRow( culture, departmentTable.Rows[0] );
            else
                return Department.Null;
        }

        public Department GetOneByUrlName( Culture culture, string urlName )
        {
            DataTable table = _categoryHelperRepository.GetOneByUrlName( culture, urlName );

            if (table.Rows.Count > 0)
                return CreateOneDepartmentFromRow( culture, table.Rows[0] );
            else
                return Department.Null;
        }

        public IList<Department> GetByParentID(
            Culture culture,
            string parentDepartmentID,
            string sortBy,
            BoolFilter isEnabled )
        {
            DataTable table = _categoryHelperRepository.GetByParentID( culture, parentDepartmentID, sortBy, isEnabled );
            DataColumn[] keys = new DataColumn[1] { table.Columns["DepartmentID"] };
            table.PrimaryKey = keys;

            return ConvertDepartmentTableToList( culture, table );
        }

        public IList<Department> GetByParentIDAndRootID(
            Culture culture,
            string parentDepartmentID,
            string rootID,
            string sortBy,
            BoolFilter isEnabled )
        {
            DataTable table = _categoryHelperRepository.GetByParentIDAndRootID( culture, parentDepartmentID, rootID, sortBy, isEnabled );

            DataColumn[] keys = new DataColumn[1] { table.Columns["DepartmentID"] };
            table.PrimaryKey = keys;

            return ConvertDepartmentTableToList( culture, table );
        }

        public IList<Department> GetByParentID(
            Culture culture,
            string parentDepartmentID,
            string sortBy,
            BoolFilter isEnabled,
            int startIndex,
            int endIndex,
            out int howManyItems )
        {
            IList<Department> full = GetByParentID( culture, parentDepartmentID, sortBy, isEnabled );

            howManyItems = full.Count;

            return GetOnlyIndex( full, startIndex, endIndex );
        }

        public IList<Department> GetByRootID(
            Culture culture,
            string rootID,
            string sortBy,
            BoolFilter isEnabled )
        {
            DataTable table = _categoryHelperRepository.GetByRootID( culture, rootID, sortBy, isEnabled );

            return ConvertDepartmentTableToList( culture, table );
        }

        public IList<Department> GetByRootIDLeafOnly(
            Culture culture,
            string rootID,
            string sortBy,
            BoolFilter isEnabled )
        {
            DataTable table = _categoryHelperRepository.GetByRootIDLeafOnly( culture, rootID, sortBy, isEnabled );

            return ConvertDepartmentTableToList( culture, table );
        }

        public IList<Department> GetByRootIDNotLeaf(
            Culture culture,
            string rootID,
            string sortBy,
            BoolFilter isEnabled )
        {
            DataTable table = _categoryHelperRepository.GetByRootIDNotLeaf( culture, rootID, sortBy, isEnabled );

            DataColumn[] keys = new DataColumn[1] { table.Columns["CategoryID"] };
            table.PrimaryKey = keys;

            return ConvertDepartmentTableToList( culture, table );
        }

        public IList<Department> GetByRootIDNoProduct(
            Culture culture,
            string rootID,
            string sortBy,
            BoolFilter isEnabled )
        {
            DataTable table = _categoryHelperRepository.GetByRootIDNoProduct( culture, rootID, sortBy, isEnabled );

            DataColumn[] keys = new DataColumn[1] { table.Columns["CategoryID"] };
            table.PrimaryKey = keys;

            return ConvertDepartmentTableToList( culture, table );
        }

        public IList<Department> GetByRootID(
            Culture culture,
            string rootID,
            string sortBy,
            BoolFilter isEnabled,
            int startIndex,
            int endIndex,
            out int howManyItems )
        {
            IList<Department> full = GetByRootID( culture, rootID, sortBy, isEnabled );

            howManyItems = full.Count;

            return GetOnlyIndex( full, startIndex, endIndex );
        }

        public IList<Department> GetRootDepartment(
            Culture culture,
            string sortBy,
            BoolFilter isEnabled )
        {
            DataTable table = _categoryHelperRepository.GetRootCategory( culture, sortBy, isEnabled );

            DataColumn[] keys = new DataColumn[1] { table.Columns["DepartmentID"] };
            table.PrimaryKey = keys;

            return ConvertDepartmentTableToList( culture, table );
        }

        public IList<Department> GetRootDepartment(
            Culture culture,
            string sortBy,
            BoolFilter isEnabled,
            int startIndex,
            int endIndex,
            out int howManyItems )
        {
            IList<Department> full = GetRootDepartment( culture, sortBy, isEnabled );

            howManyItems = full.Count;

            return GetOnlyIndex( full, startIndex, endIndex );
        }

        public IList<Department> GetByParentUrlName(
             Culture culture,
             string parentDepartmentName,
             string sortBy,
             BoolFilter isEnabled,
             int startIndex,
             int endIndex,
             out int howManyItems )
        {
            DataTable full = _categoryHelperRepository.GetByParentUrlName( culture, parentDepartmentName, sortBy, isEnabled, startIndex, endIndex );
            howManyItems = _categoryHelperRepository.GetCountByParentUrlName( culture, parentDepartmentName, sortBy, isEnabled );
            return ConvertDepartmentTableToList( culture, full );
        }

        public IList<Department> GetAll( Culture culture, string sortBy )
        {
            DataTable table = _categoryHelperRepository.GetAll( culture, sortBy );

            return ConvertDepartmentTableToList( culture, table );
        }

        public IList<Department> GetAllLeafDepartment( Culture culture, string sortBy )
        {
            DataTable table = _categoryHelperRepository.GetAllLeafOnly( culture, sortBy );

            return ConvertDepartmentTableToList( culture, table );
        }

        public IList<Department> GetAll( Culture culture, string sortBy, BoolFilter isEnabled )
        {
            DataTable table = _categoryHelperRepository.GetAll( culture, sortBy, isEnabled );

            DataColumn[] keys = new DataColumn[1] { table.Columns["DepartmentID"] };
            table.PrimaryKey = keys;
            return ConvertDepartmentTableToList( culture, table );
        }

        public IList<Department> GetAll(
            Culture culture,
            string sortBy,
            int startIndex,
            int endIndex,
            out int howManyItems )
        {
            IList<Department> full = GetAll( culture, sortBy );
            howManyItems = full.Count;
            return GetOnlyIndex( full, startIndex, endIndex );
        }

        public IList<Department> GetAllWithoutRootDepartment( Culture culture, string sortBy )
        {
            DataTable table = _categoryHelperRepository.GetAllWithoutRootCategory( culture, sortBy );

            return ConvertDepartmentTableToList( culture, table );
        }

        public IList<ILocale> GetLocales( string departmentID )
        {
            DataTable localeTable = _categoryHelperRepository.GetLocales( departmentID );

            return ConvertDepartmentLocaleTableToList( localeTable );
        }

        public IList<TableSchemaItem> GetTableSchema()
        {
            return _categoryHelperRepository.GetTableSchema();
        }

        public bool IsLeaf( string departmentID )
        {
            return _categoryHelperRepository.IsLeaf( departmentID );
        }

        public bool IsDepartmentIDNotLeaf( string parentDepartmentID )
        {
            return _categoryHelperRepository.IsNotLeaf( parentDepartmentID );
        }

        public bool IsUrlNameNotLeaf( string parentDepartmentUrlName )
        {
            return _categoryHelperRepository.IsUrlNameNotLeaf( parentDepartmentUrlName );
        }

        public void RefreshUrlName( string departmentID, string name )
        {
            _categoryHelperRepository.RefreshUrlName( departmentID, name );
        }

        public IList<Department> SearchDepartment(
            Culture culture,
            string sortBy,
            SearchFilter searchFilter
            )
        {
            DataTable full = _categoryHelperRepository.Search( culture, sortBy, searchFilter );

            DataColumn[] keys = new DataColumn[] { full.Columns["DepartmentID"] };
            full.PrimaryKey = keys;
            return ConvertDepartmentTableToList( culture, full );
        }

        public IList<Department> SearchDepartmentByRootID(
            Culture culture,
            string sortBy,
            SearchFilter searchFilter,
            string rootID
            )
        {
            DataTable full = _categoryHelperRepository.SearchByRootID( culture, sortBy, searchFilter, rootID );

            DataColumn[] keys = new DataColumn[] { full.Columns["DepartmentID"] };
            full.PrimaryKey = keys;
            return ConvertDepartmentTableToList( culture, full );
        }

        public IList<Department> SearchRootDepartment(
           Culture culture,
           string sortBy,
           SearchFilter searchFilter
           )
        {
            DataTable full = _categoryHelperRepository.SearchRoot( culture, sortBy, searchFilter );

            DataColumn[] keys = new DataColumn[] { full.Columns["DepartmentID"] };
            full.PrimaryKey = keys;
            return ConvertDepartmentTableToList( culture, full );
        }


        public IList<string> GetLeafFromDepartmentID( string parentDepartmentID, IList<string> list )
        {
            if (_categoryHelperRepository.IsNotLeaf( parentDepartmentID ))
            {
                DataTable table = DataAccess.ExecuteSelect(
                    " SELECT * FROM Department " +
                    " WHERE ParentDepartmentID = " + parentDepartmentID );

                foreach (DataRow row in table.Rows)
                    list.Add( row["DepartmentID"].ToString() );

                IList<string> newList = new List<string>();
                for (int i = 0; i <= list.Count - 1; i++)
                {
                    string departmentID = list[i];
                    if (_categoryHelperRepository.IsNotLeaf( departmentID ))
                    {
                        list.RemoveAt( i );
                        DataAccessContext.DepartmentRepository.GetLeafFromDepartmentID( departmentID, list );
                    }
                }
            }
            else
            {
                list.Add( parentDepartmentID );
            }

            return list;
        }

        public bool HasDepartmentInDiscountGroup( string discountgroupID )
        {
            return _categoryHelperRepository.HasItInDiscountGroup( discountgroupID );
        }

        public int GetCountAllWithoutRootDepartment()
        {
            return _categoryHelperRepository.GetCountAllWithoutRootCategoryDepartment();
        }

        #endregion
    }
}
