﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Data;
using System.Collections;
using Vevo.Domain.DataInterfaces;
using Vevo.Domain.GoogleFeed;
using Vevo.Shared.DataAccess;
using Vevo.Shared.Utilities;
using Vevo.Base.Data;
using Vevo.Base.Domain;

namespace Vevo.Data
{
    public class GoogleFeedTagRepositoryNoCache : IGoogleFeedTagRepository
    {
        #region Private

        private const string AllGoogleTagField = " GoogleFeedTagID, TagName, GoogleFeedTag.GoogleFeedTagTypeID, TagTypeName ";

        private GoogleFeedTag Create( GoogleFeedTag googleFeedTag )
        {
            string googleFeedTagID = DataAccess.ExecuteScalar(
                "INSERT INTO GoogleFeedTag ( TagName, GoogleFeedTagTypeID ) " +
                "VALUES ( @TagName, @GoogleFeedTagTypeID );" +
                "SELECT @@Identity; ",
                DataAccess.CreateParameterString( googleFeedTag.TagName ),
                DataAccess.CreateParameterString( googleFeedTag.GoogleFeedTagTypeID ) );

            googleFeedTag.GoogleFeedTagID = googleFeedTagID;

            return googleFeedTag;
        }

        private GoogleFeedTag Update( GoogleFeedTag googleFeedTag )
        {
            int update = DataAccess.ExecuteNonQuery(
                "UPDATE GoogleFeedTag " +
                "SET TagName = @TagName, GoogleFeedTagTypeID = @GoogleFeedTagTypeID " +
                "WHERE GoogleFeedTagID = @GoogleFeedTagID; ",
                DataAccess.CreateParameterString( googleFeedTag.TagName ),
                DataAccess.CreateParameterString( googleFeedTag.GoogleFeedTagTypeID ),
                DataAccess.CreateParameterString( googleFeedTag.GoogleFeedTagID ) );

            return googleFeedTag;
        }

        private int GetGoogleFeedTagCount()
        {
            string count = DataAccess.ExecuteScalar(
                "SELECT COUNT (GoogleFeedTagID) AS [RowCount] " +
                "FROM GoogleFeedTag LEFT JOIN " +
                "    (SELECT * FROM GoogleFeedTagType) L " +
                "    ON GoogleFeedTag.GoogleFeedTagTypeID = L.GoogleFeedTagTypeID " );
            return ConvertUtilities.ToInt32( count );
        }

        private GoogleFeedTag CreateOneFromRow( DataRow row )
        {
            GoogleFeedTag googleFeedTag = new GoogleFeedTag();
            googleFeedTag.GoogleFeedTagID = row["GoogleFeedTagID"].ToString();
            googleFeedTag.TagName = row["TagName"].ToString();
            googleFeedTag.GoogleFeedTagTypeID = row["GoogleFeedTagTypeID"].ToString();

            return googleFeedTag;
        }

        private IList<GoogleFeedTag> ConvertTableToList( DataTable table, int startIndex, int endIndex )
        {
            IList<GoogleFeedTag> list = new List<GoogleFeedTag>();

            for (int i = startIndex; i <= endIndex && i < table.Rows.Count; i++)
                list.Add( CreateOneFromRow( table.Rows[i] ) );

            return list;
        }

        private string CreateSortByText( string sortBy )
        {
            if (sortBy == null || sortBy.Trim() == "")
                return "";
            else if (sortBy.Trim().ToLower().StartsWith( "GoogleFeedTagID" ))
                return "ORDER BY GoogleFeedTag." + sortBy.Trim() + " ";
            else
                return "ORDER BY " + sortBy + " ";
        }

        private string ConvertFieldName( string fieldName )
        {
            switch (fieldName.ToLower())
            {
                case "googlefeedtagtypid":
                    return "GoogleFeedTag.GoogleFeedTagTypeID";
                default:
                    return fieldName;
            }
        }

        #endregion

        #region Public

        public GoogleFeedTag Save( GoogleFeedTag googleFeedTag )
        {
            GoogleFeedTag result = GetOne( googleFeedTag.GoogleFeedTagID );

            using (DataTransactionScope transaction = new DataTransactionScope())
            {
                if (result.IsNull)
                    result = Create( googleFeedTag );
                else
                    result = Update( googleFeedTag );

                transaction.Complete();
            }

            return result;
        }

        public IList<TableSchemaItem> GetTableSchema()
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + AllGoogleTagField +
                "FROM GoogleFeedTag LEFT JOIN " +
                "    (SELECT * FROM GoogleFeedTagType) L " +
                "    ON GoogleFeedTag.GoogleFeedTagTypeID = L.GoogleFeedTagTypeID " +
                "WHERE 1 = 0 " );

            return RepositoryUtilities.ConvertTableToSchemaItemList( table );
        }

        public GoogleFeedTag GetOne( string googleFeedTagID )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + AllGoogleTagField +
                "FROM GoogleFeedTag LEFT JOIN " +
                "    (SELECT * FROM GoogleFeedTagType) L " +
                "    ON GoogleFeedTag.GoogleFeedTagTypeID = L.GoogleFeedTagTypeID " +
                "WHERE GoogleFeedTagID = @GoogleFeedTagID;",
                DataAccess.CreateParameterString( googleFeedTagID ) );

            if (table.Rows.Count > 0)
                return CreateOneFromRow( table.Rows[0] );
            else
                return GoogleFeedTag.Null;
        }

        public IList<GoogleFeedTag> GetAll()
        {
            return GetAll( String.Empty );
        }

        public IList<GoogleFeedTag> GetAll( string sortBy )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + AllGoogleTagField +
                "FROM GoogleFeedTag LEFT JOIN " +
                "    (SELECT * FROM GoogleFeedTagType) L " +
                "    ON GoogleFeedTag.GoogleFeedTagTypeID = L.GoogleFeedTagTypeID " +
                CreateSortByText( sortBy ) + "; " );

            return ConvertTableToList( table, 0, table.Rows.Count );
        }

        public IList<GoogleFeedTag> GetAllByTagTypeID( string googleFeedTagTypeID )
        {
            return GetAllByTagTypeID( googleFeedTagTypeID, String.Empty );
        }

        public IList<GoogleFeedTag> GetAllByTagTypeID( string googleFeedTagTypeID, string sortBy )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + AllGoogleTagField +
                "FROM GoogleFeedTag LEFT JOIN " +
                "    (SELECT * FROM GoogleFeedTagType) L " +
                "    ON GoogleFeedTag.GoogleFeedTagTypeID = L.GoogleFeedTagTypeID " +
                "WHERE GoogleFeedTag.GoogleFeedTagTypeID = @GoogleFeedTagTypeID " +
                CreateSortByText( sortBy ) + "; ",
                DataAccess.CreateParameterString( googleFeedTagTypeID ) );

            return ConvertTableToList( table, 0, table.Rows.Count );
        }

        public IList<GoogleFeedTag> SearchGoogleFeedTag(
            string sortBy,
            SearchFilter searchFilter,
            int startIndex,
            int endIndex,
            out int howManyItems )
        {
            ArrayList parameterList = new ArrayList();
            SearchFilterHelperNew searchFilterHelper = new SearchFilterHelperNew();
            string optionFieldName = ConvertFieldName( searchFilter.FieldName );
            string commandSearch = searchFilterHelper.CreateCommandWithWhere( searchFilter, optionFieldName, parameterList );

            DataTable full = DataAccess.ExecuteSelect(
                "SELECT " + AllGoogleTagField +
                "FROM GoogleFeedTag LEFT JOIN " +
                "    (SELECT * FROM GoogleFeedTagType) L " +
                "    ON GoogleFeedTag.GoogleFeedTagTypeID = L.GoogleFeedTagTypeID " +
                commandSearch +
                CreateSortByText( sortBy ) + "; ",
                DataAccess.ConvertArrayListToDbParameters( parameterList ) );

            howManyItems = GetGoogleFeedTagCount();

            DataColumn[] keys = new DataColumn[] { full.Columns["GoogleFeedTagID"] };
            full.PrimaryKey = keys;
            return ConvertTableToList( full, startIndex, endIndex );
        }

        public void Delete( string googleFeedTagID )
        {
            using (DataTransactionScope transaction = new DataTransactionScope())
            {
                DataAccess.ExecuteNonQuery(
                "DELETE FROM GoogleFeedTag WHERE GoogleFeedTagID = @GoogleFeedTagID;",
                DataAccess.CreateParameterString( googleFeedTagID ) );

                transaction.Complete();
            }
        }

        #endregion
    }
}
