using System;
using System.Collections.Generic;
using System.Data;
using System.Text;
using Vevo.Data.Implementation;
using Vevo.Domain;
using Vevo.Domain.DataInterfaces;
using Vevo.Domain.Orders;
using Vevo.Shared.DataAccess;
using Vevo.Shared.Utilities;
using System.Collections;
using Vevo.Base.Data;
using Vevo.Base.Domain;

namespace Vevo.Data
{
    public class OrderItemRepositoryNoCache : IOrderItemRepository
    {
        #region Private

        private const string AllOrderItemFields =
            " OrderItemID, OrderID, ProductID, Name, Quantity, Sku, Model, " +
            "UnitPrice, GiftRegistryItemID, UploadFile, OrderItem.RecurringID, DownloadCount ";

        private OrderItemRecurringAccess _orderItemRecurringAccess = new OrderItemRecurringAccess();


        private OrderItem Create( OrderItem orderItem )
        {
            string orderItemID = DataAccess.ExecuteScalar(
                "INSERT INTO OrderItem (OrderID, ProductID, Quantity, Name, Sku, Model, UnitPrice, " +
                "GiftRegistryItemID, UploadFile, RecurringID, DownloadCount ) "+
                "VALUES ( @OrderID, @ProductID, @Quantity, " +
                " @Name , @Sku, @Model, @UnitPrice, @GiftRegistryItemID, @UploadFile, @RecurringID , "+
                " @DownloadCount) ; " +
                "SELECT @@Identity; ",
                DataAccess.CreateParameterString( orderItem.OrderID ),
                DataAccess.CreateParameterString( orderItem.ProductID ),
                DataAccess.CreateParameterInt32( orderItem.Quantity ),
                DataAccess.CreateParameterString( orderItem.Name ),
                DataAccess.CreateParameterString( orderItem.Sku ),
                DataAccess.CreateParameterString( orderItem.Model ),
                DataAccess.CreateParameterDecimalMoney( orderItem.UnitPrice ),
                DataAccess.CreateParameterString( orderItem.GiftRegistryItemID ),
                DataAccess.CreateParameterString( orderItem.UploadFile ),
                DataAccess.CreateParameterString( orderItem.RecurringID ),
                DataAccess.CreateParameterInt32(orderItem.DownloadCount)
                );
            orderItem.OrderItemID = orderItemID;
            orderItem.OrderItemRecurring.OrderItemID = orderItemID;

            if (orderItem.RecurringID != "0")
                _orderItemRecurringAccess.Create( orderItem.OrderItemRecurring );

            return orderItem;
        }

        private OrderItem Update( OrderItem orderItem )
        {
            DataAccess.ExecuteNonQuery(
                "UPDATE OrderItem " +
                "SET  Quantity = @Quantity , " +
                "Name = @Name , Sku = @Sku , UnitPrice = @UnitPrice ,  DownloadCount = @DownloadCount " +
                "WHERE OrderItemID = @OrderItemID ",

                DataAccess.CreateParameterInt32( orderItem.Quantity ),
                DataAccess.CreateParameterString( orderItem.Name ),
                DataAccess.CreateParameterString( orderItem.Sku ),
                DataAccess.CreateParameterDecimalMoney( orderItem.UnitPrice ),
                DataAccess.CreateParameterInt32(orderItem.DownloadCount),
                DataAccess.CreateParameterString( orderItem.OrderItemID )
            );
            return orderItem;
        }

        private OrderItem CreateOneFromRow( DataRow row )
        {
            OrderItem orderItem = new OrderItem();
            orderItem.OrderItemID = row["OrderItemID"].ToString();
            orderItem.OrderID = row["OrderID"].ToString();
            orderItem.ProductID = row["ProductID"].ToString();
            orderItem.Name = row["Name"].ToString();
            orderItem.Quantity = ConvertUtilities.ToInt32( row["Quantity"] );
            orderItem.Sku = row["Sku"].ToString();
            orderItem.Model = row["Model"].ToString();
            orderItem.UnitPrice = ConvertUtilities.ToDecimal( row["UnitPrice"] );
            orderItem.GiftRegistryItemID = row["GiftRegistryItemID"].ToString();
            orderItem.UploadFile = row["UploadFile"].ToString();
            orderItem.RecurringID = row["RecurringID"].ToString();
            orderItem.OrderItemRecurring = _orderItemRecurringAccess.GetOne( orderItem.OrderItemID );
            orderItem.DownloadCount = ConvertUtilities.ToInt32( row["DownloadCount"] );
            return orderItem;
        }

        private void DeleteOrderItemAndRelatedDetails( string orderItemID )
        {
            DataAccessContext.GiftCertificateRepository.DeleteByOrderItemID( orderItemID );

            DataAccess.ExecuteNonQuery(
                "DELETE " +
                "FROM OrderItem " +
                "WHERE OrderItemID = @OrderItemID; ",
                DataAccess.CreateParameterString( orderItemID )
                );
            _orderItemRecurringAccess.Delete( orderItemID );
        }

        private IList<OrderItem> ConvertTableToList( DataTable table, int startIndex, int endIndex )
        {
            IList<OrderItem> list = new List<OrderItem>();

            for (int i = startIndex; i <= endIndex && i < table.Rows.Count; i++)
                list.Add( CreateOneFromRow( table.Rows[i] ) );

            return list;
        }

        private IList<OrderItem> ConvertTableToList( DataTable table )
        {
            return ConvertTableToList( table, 0, table.Rows.Count );
        }

        private string CreateSortByText( string sortBy )
        {
            if (sortBy == null || sortBy.Trim() == "")
                return "";
            else
                return "ORDER BY " + sortBy;
        }

        #endregion


        #region Public Methods

        public OrderItem Save( OrderItem orderItem )
        {
            OrderItem result;

            if (orderItem.OrderItemID == "0")
                result = Create( orderItem );
            else
                result = Update( orderItem );

            return result;
        }

        public OrderItem GetOne( string orderItemID )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + AllOrderItemFields +
                "FROM OrderItem " +
                "WHERE OrderItemID = @OrderItemID;",
                DataAccess.CreateParameterString( orderItemID ) );

            if (table.Rows.Count > 0)
                return CreateOneFromRow( table.Rows[0] );
            else
                return null;
        }

        public void Delete( string orderItemID )
        {
            using (DataTransactionScope transaction = new DataTransactionScope())
            {
                DeleteOrderItemAndRelatedDetails( orderItemID );
                transaction.Complete();
            }
        }

        public void DeleteByOrderID( string orderID )
        {
            DataAccess.ExecuteNonQuery(
                "DELETE " +
                "FROM OrderItem " +
                "WHERE OrderID = @OrderID; ",
                DataAccess.CreateParameterString( orderID )
                );
        }

        public IList<OrderItem> GetByOrderID( string orderID )
        {
            DataTable result = DataAccess.ExecuteSelect(
                "SELECT " + AllOrderItemFields +
                "FROM OrderItem " +
                "WHERE OrderID = @OrderID " +
                "ORDER BY OrderItemID; ",
                DataAccess.CreateParameterString( orderID ) );
            return ConvertTableToList( result, 0, result.Rows.Count );
        }

        public decimal GetSubtotal( string OrderID )
        {
            string subtotal = DataAccess.ExecuteScalar(
                "SELECT SUM ( UnitPrice * Quantity ) FROM OrderItem Where OrderID = @OrderID ",
                DataAccess.CreateParameterString( OrderID )
            );
            if (String.IsNullOrEmpty( subtotal ))
                subtotal = "0";

            return ConvertUtilities.ToDecimal( subtotal );
        }

        public void UpdateRecurringItem( OrderItemRecurring orderItemRecurring )
        {
            _orderItemRecurringAccess.Update( orderItemRecurring );
        }

        public DataTable GetAllChildOrderRecurring( int orderID )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT A.OrderID, OrderItemID, GatewayPaymentStatus, RecurringStatus, RecurringPeriod FROM " +
                "( SELECT * FROM Orders WHERE ParentID = @ParentID " +
                "AND ContainsRecurring = @ContainsRecurring ) AS A LEFT JOIN (SELECT OrderItem.OrderItemID, OrderID, RecurringStatus, RecurringPeriod FROM OrderItem LEFT JOIN OrderItemRecurring ON OrderItem.OrderItemID = OrderItemRecurring.OrderItemID ) AS B ON A.OrderID = B.OrderID",
                DataAccess.CreateParameterInt32( orderID ),
                DataAccess.CreateParameterBool( true ) );
            return table;
        }

        public IList<TableSchemaItem> GetTableSchema()
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + AllOrderItemFields +
                "FROM  OrderItem " +
                "WHERE 0 = 1; " );

            return RepositoryUtilities.ConvertTableToSchemaItemList( table );
        }

        public IList<OrderItem> ExportOrderItem(
            string sortBy,
            SearchFilter searchFilter )
        {
            ArrayList parameterList = new ArrayList();
            SearchFilterHelperNew searchFilterHelper = new SearchFilterHelperNew();
            string commandsearch = searchFilterHelper.CreateCommandWithWhere( searchFilter, searchFilter.FieldName, parameterList );

            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + AllOrderItemFields +
                "FROM OrderItem " +
                commandsearch +
                CreateSortByText( sortBy ) + "; ",
                DataAccess.ConvertArrayListToDbParameters( parameterList ) );

            return ConvertTableToList( table );
        }


        #endregion
    }
}
