﻿using System.Collections.Generic;
using System.Data;
using System.Web.UI;
using Vevo.Base.Data.Caching;
using Vevo.Domain;
using Vevo.Domain.Base;
using Vevo.Domain.DataInterfaces;
using Vevo.Domain.Products;
using Vevo.Shared.Utilities;
using Vevo.Base.Domain;

namespace Vevo.Data
{
    public class SpecificationItemRepository : ISpecificationItemRepository
    {
        private const string RepositoryName = "SpecificationItemRepository";
        private const string RepositoryName1 = "GoogleFeedMappingRepository";
        private const string Product_Repository = "ProductRepository";

        private const string GetLocalesSignature = "GetLocales";
        private const string GetTableSchemaSignature = "GetTableSchema";
        private const string GetAllSignature = "GetAll";
        private const string GetOneSignature = "GetOne";
        private const string GetOneByNameSignature = "GetOneByName";
        private const string GetAllProductSpecificationValueByItemIDSignature = "GetAllProductSpecificationValueByItemID";
        private const string GetAllProductSpecificationValueByItemIDAndValueSignature = "GetAllProductSpecificationValueByItemIDAndValue";
        private const string GetBySpecificationGroupIDSignature = "GetBySpecificationGroupID";
        private const string SearchSpecificationItemSignature = "SearchSpecificationItem";

        private ICacheDependencyUtility _cacheDependency
            = new CacheDependencyUtility( new string[] { "SpecificationItem", "SpecificationItemLocale" } );

        private DataCache _dataCache = new DataCache( RepositoryName );
        private DataCache _dataCache1 = new DataCache( RepositoryName1 );
        private DataCache _productCache = new DataCache( Product_Repository );
        private SpecificationItemRepositoryNoCache _repository = new SpecificationItemRepositoryNoCache();

        #region Constructors

        public SpecificationItemRepository()
        {
        }

        public SpecificationItemRepository( ICacheDependencyUtility cacheDependency )
        {
            _cacheDependency = cacheDependency;
        }

        #endregion

        #region ISpecificationItemRepository Members

        public IList<ILocale> GetLocales( string specificationItemID )
        {
            IList<ILocale> list = (IList<ILocale>) _dataCache.GetValue( GetLocalesSignature, _cacheDependency, specificationItemID );

            if (list == null)
            {
                list = _repository.GetLocales( specificationItemID );
                _dataCache.SetValue( GetLocalesSignature, list, specificationItemID );
            }

            return list;
        }

        public IList<TableSchemaItem> GetTableSchema()
        {
            IList<TableSchemaItem> list = (IList<TableSchemaItem>) _dataCache.GetValue(
                  GetTableSchemaSignature, _cacheDependency
                  );

            if (list == null)
            {
                list = _repository.GetTableSchema();
                _dataCache.SetValue(
                    GetTableSchemaSignature,
                    list
                    );
            }

            return list;
        }

        public SpecificationItem Save( SpecificationItem specificationItem )
        {
            // Need to write to database first before clearing cache.
            // Otherwise, cache may remember incorrect values before saving.
            SpecificationItem newSpecificationItem = _repository.Save( specificationItem );
            _dataCache.ClearRepositoryCache();
            _dataCache1.ClearRepositoryCache();
            _productCache.ClearRepositoryCache();
            return newSpecificationItem;
        }

        public void Delete( string specificationItemID )
        {
            _repository.Delete( specificationItemID );
            _dataCache.ClearRepositoryCache();
            _dataCache1.ClearRepositoryCache();
            _productCache.ClearRepositoryCache();
        }

        public void DeleteLocalesByCultureID( string cultureID )
        {
            _repository.DeleteLocalesByCultureID( cultureID );
            _dataCache.ClearRepositoryCache();
            _dataCache1.ClearRepositoryCache();
            _productCache.ClearRepositoryCache();
        }

        public void DeleteBySpecificationGroupID( string specificationGroupID )
        {
            _repository.DeleteBySpecificationGroupID( specificationGroupID );
            _dataCache.ClearRepositoryCache();
            _dataCache1.ClearRepositoryCache();
            _productCache.ClearRepositoryCache();
        }

        public DataTable GetAll( Culture culture )
        {
            DataTable result = (DataTable) _dataCache.GetValue(
                GetAllSignature, _cacheDependency, culture.CultureID );

            if (result == null)
            {
                result = _repository.GetAll( culture );
                _dataCache.SetValue( GetAllSignature, result, culture.CultureID );
            }

            return result;
        }

        public IList<ProductSpecification> GetAllProductSpecificationValueByItemID( string specificationItemID )
        {
            IList<ProductSpecification> list = (IList<ProductSpecification>) _dataCache.GetValue(
               GetAllProductSpecificationValueByItemIDSignature, _cacheDependency, specificationItemID );

            if (list == null)
            {
                list = _repository.GetAllProductSpecificationValueByItemID( specificationItemID );
                _dataCache.SetValue( GetAllProductSpecificationValueByItemIDSignature, list, specificationItemID );
            }

            return list;
        }

        public IList<ProductSpecification> GetAllProductSpecificationValueByItemIDAndValue( string specificationItemID, string specificationItemValue )
        {
            IList<ProductSpecification> list = (IList<ProductSpecification>) _dataCache.GetValue(
               GetAllProductSpecificationValueByItemIDAndValueSignature, _cacheDependency, specificationItemID, specificationItemValue );

            if (list == null)
            {
                list = _repository.GetAllProductSpecificationValueByItemIDAndValue( specificationItemID, specificationItemValue );
                _dataCache.SetValue( GetAllProductSpecificationValueByItemIDAndValueSignature, list, specificationItemID, specificationItemValue );
            }

            return list;
        }

        public SpecificationItem GetOne( Culture culture, string specificationItemID )
        {
            SpecificationItem result = (SpecificationItem) _dataCache.GetValue(
                GetOneSignature, _cacheDependency, culture.CultureID, specificationItemID );

            if (result == null)
            {
                result = _repository.GetOne( culture, specificationItemID );
                _dataCache.SetValue( GetOneSignature, result, culture.CultureID, specificationItemID );
            }

            return result;
        }

        public SpecificationItem GetOneByName( Culture culture, string name )
        {
            SpecificationItem result = (SpecificationItem) _dataCache.GetValue(
                GetOneSignature, _cacheDependency, culture.CultureID, name );

            if (result == null)
            {
                result = _repository.GetOneByName( culture, name );
                _dataCache.SetValue( GetOneSignature, result, culture.CultureID, name );
            }

            return result;
        }

        public IList<SpecificationItem> GetBySpecificationGroupID( Culture culture, string specificationGroupID )
        {
            IList<SpecificationItem> list = (IList<SpecificationItem>) _dataCache.GetValue(
               GetBySpecificationGroupIDSignature, _cacheDependency, culture, specificationGroupID );

            if (list == null)
            {
                list = _repository.GetBySpecificationGroupID( culture, specificationGroupID );
                _dataCache.SetValue( GetBySpecificationGroupIDSignature, list, culture, specificationGroupID );
            }

            return list;
        }

        public IList<SpecificationItem> SearchSpecificationItem(
            Culture culture,
            string specificationGroupID,
            string sortBy,
            SearchFilter searchfilter,
            int startIndex,
            int endIndex,
            out int howManyItems )
        {
            howManyItems = 0;
            Pair result = (Pair) _dataCache.GetValue(
                SearchSpecificationItemSignature,
                _cacheDependency,
                culture.CultureID,
                specificationGroupID,
                sortBy,
                searchfilter.ToString(),
                startIndex,
                endIndex
                );

            if (result == null)
            {
                result = new Pair();
                result.First = _repository.SearchSpecificationItem(
                    culture, specificationGroupID, sortBy, searchfilter, startIndex, endIndex, out howManyItems );

                result.Second = howManyItems;
                _dataCache.SetValue(
                    SearchSpecificationItemSignature,
                    result,
                    culture.CultureID,
                    specificationGroupID,
                    sortBy,
                    searchfilter.ToString(),
                    startIndex,
                    endIndex );
            }

            howManyItems = ConvertUtilities.ToInt32( result.Second );
            return (IList<SpecificationItem>) result.First;
        }

        #endregion
    }
}
