using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Configuration;
using System.Data;
using Vevo;
using Vevo.Domain.Discounts;
using Vevo.Domain.Payments;
using Vevo.Domain.Shipping;
using Vevo.Domain.Users;
using Vevo.Shared.Utilities;
using Vevo.Domain.Stores;
using Vevo.Base.Domain;

namespace Vevo.Domain.Orders
{
    /// <summary>
    /// Summary description for CheckoutDetails
    /// </summary>
    public class CheckoutDetails
    {
        #region Public Constances

        // Custom parameter names
        public const string PayPalProExpress_TokenID = "PayPalProExpressTokenID";
        public const string PayPalProExpress_PayerID = "PayPalProExpressPayerID";
        public const string CardIssue = "CardIssue";
        public const string CardStart = "CardStart";

        #endregion


        #region Private

        private Address _billingAddress;
        private ShippingAddress _shippingAddress;

        private string _email;
        private string _customerComments;

        private Coupon _coupon = Coupon.Null;

        private ShippingMethod _shippingMethod = ShippingMethod.Null;
        private bool _showShippingAddress;

        private GiftCertificate _giftCertificate = GiftCertificate.Null;

        private PaymentMethod _paymentMethod = PaymentMethod.Null;
        private PaymentOption _paymentOption = PaymentOption.Null;

        // Credit card information
        private string _creditCardType;
        private string _crediCardNumber;
        private string _cvv2;
        private int _expirationMonth;
        private int _expirationYear;

        private string _giftRegistryID;

        // This is a temporary mapping until Gift Registry uses Cart and CartItem database tables
        private NameValueCollection _cartItemIDToGiftRegistryIDMap = new NameValueCollection();

        private Dictionary<string, object> _customParameters;

        private string _paymentToken;

        // Tax Exempt information
        private bool _isTaxExempt;
        private string _taxExemptID;
        private string _taxExemptCountry;
        private string _taxExemptState;

        private bool _isCreatedByAdmin = false;
        private string _storeID;
        private decimal _redeemPrice;
        private int _redeemPoint;

        private string _estimatedShippingSelect;

        private void PopulateOrder( Order order, Currency currency, OrderAmount orderAmount, string visitorIP )
        {
            order.Shipping = ShippingAddress;

            order.CustomerComments = CustomerComments;
            order.PaymentMethod = PaymentMethod.FullName;
            order.PONumber = PaymentMethod.PONumber;
            order.ShippingMethod = ShippingMethod.Name;
            order.Status = DataAccessContext.Configurations.GetValueList( "OrderStatusList" )[0];
            order.IPAddress = visitorIP;

            order.CouponID = Coupon.CouponID;
            order.BaseCurrencyCode = DataAccessContext.Configurations.GetValue( "BaseWebsiteCurrency" );
            order.UserCurrencyCode = currency.CurrencyCode;
            order.UserConversionRate = currency.ConversionRate;
            order.ShippingID = ShippingMethod.ShippingID;
            order.GiftCertificateCode = GiftCertificate.GiftCertificateCode;
            order.TrackingMethod = ShippingMethod.GetOnlineTrackingName();
            order.GiftRegistryID = (String.IsNullOrEmpty( GiftRegistryID )) ? "0" : GiftRegistryID;
            order.ShowShippingAddress = ShowShippingAddress;

            order.IsTaxExempt = IsTaxExempt;
            order.TaxExemptID = TaxExemptID;
            order.TaxExemptCountry = TaxExemptCountry;
            order.TaxExemptState = TaxExemptState;
            order.RedeemPoint = RedeemPoint;
            order.RedeemPrice = RedeemPrice;
            order.IsPointApplied = true;
            order.IsSubscriptionApplied = false;
            Store store = DataAccessContext.StoreRepository.GetOne( order.StoreID );

            if (DataAccessContext.Configurations.GetBoolValue( "PointSystemEnabled", store ))
            {
                if (RedeemPrice == 0 && RedeemPoint == 0 && ConvertUtilities.ToInt32( order.CustomerID ) != 0)
                {
                    order.IsPointApplied = false;
                    OrderCalculator calculator = new OrderCalculator();
                    order.PointEarned = calculator.GetPointFromPrice( (orderAmount.Subtotal - orderAmount.Discount), ConvertUtilities.ToDecimal( DataAccessContext.Configurations.GetValue( "RewardPoints", store ) ) );
                }
            }


            if (!PaymentMethod.IsNull)
            {
                if (PaymentMethod.PaymentOption.IsCreditCardNumberSaved)
                {
                    order.PaymentToken = PaymentToken;
                }
            }

            order.SetOrderAmount( orderAmount );
        }

        #endregion


        #region Constructors

        public CheckoutDetails()
        {
            _customParameters = new Dictionary<string, object>();
            _billingAddress = new Address();
            _shippingAddress = new ShippingAddress();

            Clear();
        }

        #endregion


        #region Public Properties

        public string Email
        {
            get { return _email; }
            set { _email = value; }
        }

        public string CustomerComments
        {
            get { return _customerComments; }
            set { _customerComments = value; }
        }

        public Coupon Coupon
        {
            get { return _coupon; }
            set { _coupon = value; }
        }

        public decimal RedeemPrice
        {
            get { return _redeemPrice; }
            set { _redeemPrice = value; }
        }

        public int RedeemPoint
        {
            get { return _redeemPoint; }
            set { _redeemPoint = value; }
        }

        public ShippingMethod ShippingMethod
        {
            get { return _shippingMethod; }
            set { _shippingMethod = value; }
        }

        public bool ShowShippingAddress
        {
            get { return _showShippingAddress; }
            set { _showShippingAddress = value; }
        }

        public GiftCertificate GiftCertificate
        {
            get { return _giftCertificate; }
            set { _giftCertificate = value; }
        }

        public string CreditCardType
        {
            get { return _creditCardType; }
            set { _creditCardType = value; }
        }

        public string CreditCardNumber
        {
            get { return _crediCardNumber; }
            set { _crediCardNumber = value; }
        }

        public string Cvv2
        {
            get { return _cvv2; }
            set { _cvv2 = value; }
        }

        public int ExpirationMonth
        {
            get { return _expirationMonth; }
            set { _expirationMonth = value; }
        }

        public int ExpirationYear
        {
            get { return _expirationYear; }
            set { _expirationYear = value; }
        }

        public string GiftRegistryID
        {
            get { return _giftRegistryID; }
            set { _giftRegistryID = value; }
        }

        // This is a temporary mapping until Gift Registry uses Cart and CartItem database tables
        public NameValueCollection CartItemIDToGiftRegistryIDMap
        {
            get { return _cartItemIDToGiftRegistryIDMap; }
        }

        public PaymentOption PaymentOption
        {
            get { return _paymentOption; }
            set { _paymentOption = value; }
        }

        public PaymentMethod PaymentMethod
        {
            get { return _paymentMethod; }
            set { _paymentMethod = value; }
        }

        public Dictionary<string, object> CustomParameters
        {
            get { return _customParameters; }
        }

        public Address BillingAddress
        {
            get { return _billingAddress; }
            set { _billingAddress = value; }
        }

        public ShippingAddress ShippingAddress
        {
            get { return _shippingAddress; }
            set { _shippingAddress = value; }
        }

        public string PaymentToken
        {
            get { return _paymentToken; }
            set { _paymentToken = value; }
        }

        public bool IsTaxExempt
        {
            get { return _isTaxExempt; }
            set { _isTaxExempt = value; }
        }

        public string TaxExemptID
        {
            get { return _taxExemptID; }
            set { _taxExemptID = value; }
        }

        public string TaxExemptCountry
        {
            get { return _taxExemptCountry; }
            set { _taxExemptCountry = value; }
        }

        public string TaxExemptState
        {
            get { return _taxExemptState; }
            set { _taxExemptState = value; }
        }

        public bool IsCreatedByAdmin
        {
            get { return _isCreatedByAdmin; }
            set { _isCreatedByAdmin = value; }
        }

        public string StoreID
        {
            get { return _storeID; }
            set { _storeID = value; }
        }

        public string EstimatedShippingSelect
        {
            get { return _estimatedShippingSelect; }
            set { _estimatedShippingSelect = value; }
        }

        #endregion


        #region Public Methods

        public void Clear()
        {
            _billingAddress = new Address();
            _shippingAddress = new ShippingAddress();
            _showShippingAddress = true;
            _shippingMethod = ShippingMethod.Null;
            _customerComments = String.Empty;
            _creditCardType = String.Empty;
            _crediCardNumber = String.Empty;
            _cvv2 = String.Empty;
            _expirationMonth = 0;
            _expirationYear = 0;
            _giftRegistryID = String.Empty;
            _paymentMethod = PaymentMethod.Null;
            _customParameters.Clear();
            _paymentToken = String.Empty;
            _isTaxExempt = false;
            _taxExemptID = String.Empty;
            _taxExemptCountry = String.Empty;
            _taxExemptState = String.Empty;
            _isCreatedByAdmin = false;
        }

        public Order CreateOrder(
            Customer customer,
            Currency currency,
            OrderAmount orderAmount,
            string visitorIP,
            IStoreRetriever storeRetriever )
        {
            Order order = new Order();
            order.CustomerID = customer.CustomerID;
            order.UserName = customer.UserName;
            order.Billing = BillingAddress;
            order.Email = customer.Email;
            order.StoreID = storeRetriever.GetCurrentStoreID();
            PopulateOrder( order, currency, orderAmount, visitorIP );

            return order;
        }

        public Order CreateOrderAnonymous(
            string userName,
            Address billingAddress,
            string email,
            Currency currency,
            OrderAmount orderAmount,
            string visitorIP,
            IStoreRetriever storeRetriever
            )
        {
            Order order = new Order();
            order.CustomerID = Customer.Null.CustomerID;
            order.UserName = userName;
            order.Billing = billingAddress;
            order.Email = email;
            order.StoreID = storeRetriever.GetCurrentStoreID();

            PopulateOrder( order, currency, orderAmount, visitorIP );

            return order;
        }

        public bool ContainsGiftRegistry()
        {
            return (ConvertUtilities.ToInt32( GiftRegistryID ) != 0);
        }

        public void SetCouponID( string couponID )
        {
            Coupon = DataAccessContext.CouponRepository.GetOne( couponID );
        }

        public void SetGiftCertificate( string giftCertificateCode )
        {
            GiftCertificate =
                DataAccessContext.GiftCertificateRepository.GetOne( giftCertificateCode );
        }

        public void SetCustomerComments( string customerComments )
        {
            CustomerComments = customerComments;
        }

        public void SetShippingDetails( ShippingAddress shippingAddress )
        {
            ShippingAddress = shippingAddress;
        }

        public void SetPaymentMethod( PaymentMethod paymentMethod )
        {
            PaymentMethod = paymentMethod;
        }

        public void SetGiftRegistryID( string giftRegistryID )
        {
            GiftRegistryID = giftRegistryID;
        }

        public void SetShipping( ShippingMethod shipping )
        {
            ShippingMethod = shipping;
        }

        public void SetShowShippingAddress( bool showShippingAddress )
        {
            ShowShippingAddress = showShippingAddress;
        }

        public void SetBillingDetails(
            Address billingAddress,
            string email )
        {
            BillingAddress = billingAddress;
            Email = email;
        }

        public void SetCreditCardDetails(
            string creditCardType, string number, string cvv2,
            int expMonth, int expYear )
        {
            CreditCardType = creditCardType;
            CreditCardNumber = number;
            Cvv2 = cvv2;
            ExpirationMonth = expMonth;
            ExpirationYear = expYear;
        }

        public void SetTaxExempt( bool isTaxExempt, string taxExemptID, string taxExemptCountry, string taxExemptState )
        {
            IsTaxExempt = isTaxExempt;
            TaxExemptID = taxExemptID;
            TaxExemptCountry = taxExemptCountry;
            TaxExemptState = taxExemptState;
        }

        #endregion

    }
}
