using System;
using System.Collections.Generic;
using System.Text;
using Vevo.Domain.Orders;
using Vevo.Domain.Payments;
using Vevo.Domain.Users;
using Vevo.Shared.Utilities;

namespace Vevo.Domain.Products
{
    public class GiftCertificateProduct : Product
    {
        #region Public Enum

        public enum ExpireTypeEnum
        {
            None,
            FixedDate,
            RollingDate
        }

        #endregion


        #region Private

        private decimal _giftAmount;
        private ExpireTypeEnum _expireType;
        private DateTime _fixedExpireDate;
        private int _numberOfDays;
        private bool _isElectronic;


        private bool IsGiftCertificateCodeExisting( string giftCertificateCode )
        {
            GiftCertificate giftCertificate = DataAccessContext.GiftCertificateRepository.GetOne( giftCertificateCode );
            return !giftCertificate.IsNull;
        }

        private string GenerateGiftCertificateCode()
        {
            string giftCertificateCode = RandomUtilities.RandomLettersOrDigits( 10 );
            if (IsGiftCertificateCodeExisting( giftCertificateCode ))
                giftCertificateCode = GenerateGiftCertificateCode();

            return giftCertificateCode;
        }

        private bool IsUrlParameterExisting( string urlParameter )
        {
            return !String.IsNullOrEmpty(
                DataAccessContext.GiftCertificateRepository.GetCodeFromUrlParameter( urlParameter ) );
        }

        private string GenerateUrlParameter()
        {
            string urlParameter = RandomUtilities.RandomLettersOrDigits( 16 );
            if (IsUrlParameterExisting( urlParameter ))
                urlParameter = GenerateUrlParameter();

            return urlParameter;
        }

        private bool GetIsExpirable()
        {
            return ExpireType != ExpireTypeEnum.None;
        }

        private DateTime GetExpireDate()
        {
            switch (ExpireType)
            {
                case ExpireTypeEnum.FixedDate:
                    return FixedExpireDate;

                case ExpireTypeEnum.RollingDate:
                    return DateTime.Today.AddDays( NumberOfDays );

                default:
                    return DateTime.Today;
            }
        }

        private decimal GetGiftValue( decimal userEnteredAmount )
        {
            if (IsFixedPrice)
                return GiftAmount;
            else
                return userEnteredAmount;
        }

        #endregion


        #region Constructors

        public GiftCertificateProduct( Culture defaultCulture )
            : base( defaultCulture )
        {
            IsGiftCertificate = true;
            FixedExpireDate = DateTime.Today;
        }

        #endregion


        #region Public Properties

        public decimal GiftAmount { get { return _giftAmount; } set { _giftAmount = value; } }
        public ExpireTypeEnum ExpireType { get { return _expireType; } set { _expireType = value; } }
        public DateTime FixedExpireDate { get { return _fixedExpireDate; } set { _fixedExpireDate = value; } }
        public int NumberOfDays { get { return _numberOfDays; } set { _numberOfDays = value; } }
        public bool IsElectronic { get { return _isElectronic; } set { _isElectronic = value; } }

        #endregion


        #region Public Methods

        // Gift certificates do not take wholesale discount
        public override decimal GetCheckoutPrice( WholesaleStatus wholesaleStatus, decimal userEnteredAmount ,string storeID)
        {
            ProductPrice productPrice = GetProductPrice( storeID );
            if (IsFixedPrice)
                return productPrice.Price;
            else
                return userEnteredAmount;
        }

        public override bool RequiresUserInput()
        {
            if (!IsElectronic || !IsFixedPrice)
                return true;
            else
                return base.RequiresUserInput();
        }

        public GiftCertificate GenerateGiftCertificate( string orderItemID, CartItemGiftDetails giftDetails )
        {
            GiftCertificate giftCertificate = new GiftCertificate();
            giftCertificate.GiftCertificateCode = GenerateGiftCertificateCode();
            giftCertificate.ProductID = ProductID;
            giftCertificate.OrderItemID = orderItemID;
            giftCertificate.GiftValue = GetGiftValue( giftDetails.GiftValue );
            giftCertificate.RemainValue = GetGiftValue( giftDetails.GiftValue );
            giftCertificate.UrlParameter = GenerateUrlParameter();
            giftCertificate.Recipient = giftDetails.Recipient;
            giftCertificate.PersonalNote = giftDetails.PersonalNote;
            giftCertificate.IsExpirable = GetIsExpirable();
            giftCertificate.ExpireDate = GetExpireDate();
            giftCertificate.NeedPhysical = giftDetails.NeedPhysical;
            giftCertificate.IsActive = true;

            return giftCertificate;
        }

        #endregion

    }

}
